@@ -93,6 +93,8 @@ inline static gint
 model_to_view_row(ETableItem *eti, int row)
 {
 	int i;
+	if (row == -1)
+		return -1;
 	if (eti->uses_source_model) {
 		ETableSubset *etss = E_TABLE_SUBSET(eti->table_model);
 		if (eti->row_guess >= 0 && eti->row_guess < etss->n_map) {
@@ -114,9 +116,10 @@ view_to_model_row(ETableItem *eti, int row)
 {
 	if (eti->uses_source_model) {
 		ETableSubset *etss = E_TABLE_SUBSET(eti->table_model);
-		if (row >= 0 && row < etss->n_map)
+		if (row >= 0 && row < etss->n_map) {
+			eti->row_guess = row;
 			return etss->map_table[row];
-		else
+		} else
 			return -1;
 	} else
 		return row;
@@ -126,6 +129,8 @@ inline static gint
 model_to_view_col(ETableItem *eti, int col)
 {
 	int i;
+	if (col == -1)
+		return -1;
 	for (i = 0; i < eti->cols; i++) {
 		ETableCol *ecol = e_table_header_get_column (eti->header, i);
 		if (ecol->col_idx == col)
@@ -689,6 +694,10 @@ e_table_item_row_diff (ETableItem *eti, int start_row, int end_row)
 	int height_extra = eti->horizontal_draw_grid ? 1 : 0;
 
 	total = 0;
+	if (start_row < 0)
+		start_row = 0;
+	if (end_row > eti->rows)
+		end_row = eti->rows;
 
 	for (row = start_row; row < end_row; row++)
 		total += eti_row_height (eti, row) + height_extra;
@@ -1147,6 +1156,8 @@ eti_init (GnomeCanvasItem *item)
 	eti->width                     = 0;
 	eti->minimum_width             = 0;
 
+	eti->click_count               = 0;
+
 	eti->height_cache              = NULL;
 	eti->height_cache_idle_id      = 0;
 	eti->height_cache_idle_count   = 0;
@@ -1497,7 +1508,7 @@ eti_point (GnomeCanvasItem *item, double x, double y, int cx, int cy,
 }
 
 static gboolean
-find_cell (ETableItem *eti, double x, double y, int *col_res, int *row_res, double *x1_res, double *y1_res)
+find_cell (ETableItem *eti, double x, double y, int *view_col_res, int *view_row_res, double *x1_res, double *y1_res)
 {
 	const int cols = eti->cols;
 	const int rows = eti->rows;
@@ -1509,8 +1520,8 @@ find_cell (ETableItem *eti, double x, double y, int *col_res, int *row_res, doub
 	/* FIXME: this routine is inneficient, fix later */
 
 	if (eti->grabbed_col >= 0 && eti->grabbed_row >= 0) {
-		*col_res = eti->grabbed_col;
-		*row_res = eti->grabbed_row;
+		*view_col_res = eti->grabbed_col;
+		*view_row_res = eti->grabbed_row;
 		*x1_res = x - eti->x1 - e_table_header_col_diff (eti->header, 0, eti->grabbed_col);
 		*y1_res = y - eti->y1 - e_table_item_row_diff (eti, 0, eti->grabbed_row);
 		return TRUE;
@@ -1545,10 +1556,10 @@ find_cell (ETableItem *eti, double x, double y, int *col_res, int *row_res, doub
 		if (y <= y2)
 			break;
 	}
-	*col_res = col;
+	*view_col_res = col;
 	if (x1_res)
 		*x1_res = x - x1;
-	*row_res = row;
+	*view_row_res = row;
 	if (y1_res)
 		*y1_res = y - y1;
 	return TRUE;
@@ -1727,6 +1738,15 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 			if (return_val)
 				return TRUE;
 
+			gtk_object_get(GTK_OBJECT(eti->selection),
+				       "cursor_row", &cursor_row,
+				       "cursor_col", &cursor_col,
+				       NULL);
+
+			if (cursor_row != view_to_model_row(eti, row) || cursor_col != view_to_model_col(eti, col)) {
+				eti->click_count = 0;
+			}
+
 			e_selection_model_do_something(E_SELECTION_MODEL (eti->selection), view_to_model_row(eti, row), view_to_model_col(eti, col), button.state);
 
 			gtk_object_get(GTK_OBJECT(eti->selection),
@@ -1736,6 +1756,9 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 
 			if (cursor_row == view_to_model_row(eti, row) && cursor_col == view_to_model_col(eti, col)){
 
+				eti->click_count ++;
+				eti->row_guess = row;
+
 				if ((!eti_editing(eti)) && e_table_model_is_cell_editable(eti->table_model, col, row)) {
 					e_table_item_enter_edit (eti, col, row);
 				}
@@ -1820,32 +1843,49 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 
 	case GDK_2BUTTON_PRESS: {
 		int col, row;
+#if 0
+		double x1, y1;
+#endif
 		GdkEventButton button;
 
 		if (e->button.button == 5 ||
 		    e->button.button == 4)
 			return FALSE;
 
-		gnome_canvas_item_w2i (item, &e->button.x, &e->button.y);
+		/*
+		 * click_count is so that if you click on two
+		 * different rows we don't send a double click signal.
+		 */
+
+		if (eti->click_count >= 2) {
+
+			gnome_canvas_item_w2i (item, &e->button.x, &e->button.y);
 
 #if 0
-		if (!find_cell (eti, e->button.x, e->button.y, &col, &row, &x1, &y1))
-			return TRUE;
+			if (!find_cell (eti, e->button.x, e->button.y, &current_col, &current_row, &x1, &y1))
+				return TRUE;
 #endif
-		gtk_object_get(GTK_OBJECT(eti->selection),
-			       "cursor_row", &row,
-			       "cursor_col", &col,
-			       NULL);
+
+			gtk_object_get(GTK_OBJECT(eti->selection),
+				       "cursor_row", &row,
+				       "cursor_col", &col,
+				       NULL);
+
+			button.x -= e_table_header_col_diff (eti->header, 0, model_to_view_col (eti, col));
+			button.y -= e_table_item_row_diff (eti, 0, model_to_view_row (eti, row));
 
 #if 0
-		button = *(GdkEventButton *)e;
-		button.x = x1;
-		button.y = y1;
+			button = *(GdkEventButton *)e;
+			button.x = x1;
+			button.y = y1;
 #endif
 
-		gtk_signal_emit (GTK_OBJECT (eti), eti_signals [DOUBLE_CLICK],
-				 row, col, &button);
-		d(g_print("Double click\n"));
+			if (row != -1 && col != -1) {
+				gtk_signal_emit (GTK_OBJECT (eti), eti_signals [DOUBLE_CLICK],
+						 row, col, &button);
+				d(g_print("Double click\n"));
+			}
+		}
 		break;
 	}
 	case GDK_MOTION_NOTIFY: {
@@ -2016,27 +2056,51 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 			handled = FALSE;
 			break;
 		}
+
 		if (!handled) {
-			if (!eti_editing (eti)){
-				gint col, row;
-				row = model_to_view_row(eti, cursor_row);
-				col = model_to_view_col(eti, cursor_col);
-				if (col != -1 && row != -1 && e_table_model_is_cell_editable(eti->table_model, col, row)) {
-					e_table_item_enter_edit (eti, col, row);
+			switch (e->key.keyval) {
+			case GDK_Scroll_Lock:
+			case GDK_Sys_Req:
+			case GDK_Shift_L:
+			case GDK_Shift_R:
+			case GDK_Control_L:
+			case GDK_Control_R:
+			case GDK_Caps_Lock:
+			case GDK_Shift_Lock:
+			case GDK_Meta_L:
+			case GDK_Meta_R:
+			case GDK_Alt_L:
+			case GDK_Alt_R:
+			case GDK_Super_L:
+			case GDK_Super_R:
+			case GDK_Hyper_L:
+			case GDK_Hyper_R:
+			case GDK_ISO_Lock:
+				break;
+
+			default:
+				if (!eti_editing (eti)){
+					gint col, row;
+					row = model_to_view_row(eti, cursor_row);
+					col = model_to_view_col(eti, cursor_col);
+					if (col != -1 && row != -1 && e_table_model_is_cell_editable(eti->table_model, col, row)) {
+						e_table_item_enter_edit (eti, col, row);
+					}
 				}
-			}
-			if (!eti_editing (eti)){
-				gtk_signal_emit (GTK_OBJECT (eti), eti_signals [KEY_PRESS],
-						 model_to_view_row(eti, cursor_row), cursor_col, e, &return_val);
-				if (!return_val)
-					e_selection_model_key_press(E_SELECTION_MODEL (eti->selection), (GdkEventKey *) e);
-			} else {
-				ecell_view = eti->cell_views [eti->editing_col];
-				return_val = eti_e_cell_event (eti, ecell_view, e, e->key.time,
-							       view_to_model_col(eti, eti->editing_col),
-							       eti->editing_col, eti->editing_row, E_CELL_EDITING);
-				if (!return_val)
-					e_selection_model_key_press(E_SELECTION_MODEL (eti->selection), (GdkEventKey *) e);
+				if (!eti_editing (eti)){
+					gtk_signal_emit (GTK_OBJECT (eti), eti_signals [KEY_PRESS],
+							 model_to_view_row(eti, cursor_row), cursor_col, e, &return_val);
+					if (!return_val)
+						e_selection_model_key_press(E_SELECTION_MODEL (eti->selection), (GdkEventKey *) e);
+				} else {
+					ecell_view = eti->cell_views [eti->editing_col];
+					return_val = eti_e_cell_event (eti, ecell_view, e, e->key.time,
+								       view_to_model_col(eti, eti->editing_col),
+								       eti->editing_col, eti->editing_row, E_CELL_EDITING);
+					if (!return_val)
+						e_selection_model_key_press(E_SELECTION_MODEL (eti->selection), (GdkEventKey *) e);
+				}
+				break;
 			}
 		}
 		eti->in_key_press = FALSE;
