@@ -49,6 +49,8 @@
 #include "e-text.h"
 #include "e-entry.h"
 
+#define MOVE_RIGHT_AND_UP 0
+
 #define EVIL_POINTER_WARPING_HACK
 
 #ifdef EVIL_POINTER_WARPING_HACK
@@ -56,7 +58,6 @@
 #endif
 
 #define MIN_ENTRY_WIDTH  150
-#define INNER_BORDER     2
 
 #define PARENT_TYPE gtk_table_get_type ()
 
@@ -141,8 +142,6 @@ static void
 canvas_size_allocate (GtkWidget *widget, GtkAllocation *alloc,
 		      EEntry *entry)
 {
-	gint xthick;
-	gint ythick;
 	gnome_canvas_set_scroll_region (entry->canvas,
 					0, 0, alloc->width, alloc->height);
 	gtk_object_set (GTK_OBJECT (entry->item),
@@ -150,26 +149,18 @@ canvas_size_allocate (GtkWidget *widget, GtkAllocation *alloc,
 			"clip_height", (double) (alloc->height),
 			NULL);
 
-	if (entry->priv->draw_borders) {
-		xthick = 0;
-		ythick = 0;
-	} else {
-		xthick = widget->style->klass->xthickness;
-		ythick = widget->style->klass->ythickness;
-	}
-
 	switch (entry->priv->justification) {
 	case GTK_JUSTIFY_RIGHT:
 		e_canvas_item_move_absolute(GNOME_CANVAS_ITEM(entry->item),
-					    alloc->width - xthick, ythick);
+					    alloc->width, 0);
 		break;
 	case GTK_JUSTIFY_CENTER:
 		e_canvas_item_move_absolute(GNOME_CANVAS_ITEM(entry->item),
-					    alloc->width / 2, ythick);
+					    alloc->width / 2, 0);
 		break;
 	default:
 		e_canvas_item_move_absolute(GNOME_CANVAS_ITEM(entry->item),
-					    xthick, ythick);
+					    0, 0);
 		break;
 	}
 }
@@ -178,21 +169,23 @@ static void
 canvas_size_request (GtkWidget *widget, GtkRequisition *requisition,
 		     EEntry *entry)
 {
-	int border;
+	int xthick, ythick;
 	
 	g_return_if_fail (widget != NULL);
 	g_return_if_fail (GNOME_IS_CANVAS (widget));
 	g_return_if_fail (requisition != NULL);
 
-	if (entry->priv->draw_borders)
-		border = INNER_BORDER;
-	else
-		border = 0;
+	if (entry->priv->draw_borders) {
+		xthick = 2 * widget->style->klass->xthickness;
+		ythick = 2 * widget->style->klass->ythickness;
+	} else {
+		xthick = ythick = 0;
+	}
 	
-	requisition->width = MIN_ENTRY_WIDTH + (widget->style->klass->xthickness + border) * 2;
-	requisition->height = (widget->style->font->ascent +
+	requisition->width = 2 + MIN_ENTRY_WIDTH + xthick;
+	requisition->height = (2 + widget->style->font->ascent +
 			       widget->style->font->descent +
-			       (widget->style->klass->ythickness + border) * 2);
+			       ythick);
 }
 
 static gint
@@ -294,6 +287,7 @@ e_entry_init (GtkObject *object)
 		"draw_background", TRUE,
 		"max_lines", 1,
 		"editable", TRUE,
+		"allow_newlines", FALSE,
 		NULL));
 
 	gtk_signal_connect (GTK_OBJECT (entry->item),
@@ -471,13 +465,17 @@ e_entry_show_popup (EEntry *entry, gboolean visible)
 		x = xo + dim->x;
 		y = yo + dim->height + dim->y;
 
+#if MOVE_RIGHT_AND_UP
 		/* Put our popup slightly to the right and up, to try to give a visual cue that this popup
 		 is tied to this entry.  Otherwise one-row popups can sort of "blend" with an entry
 		 directly below. */
 		fudge = MAX (dim->height/10, 3); /* just in case we are using a really big font, etc. */
 		x += 2*fudge;
 		y -= fudge;
-
+#else
+		fudge = 1;
+		y -= fudge;
+#endif
 		gtk_widget_set_uposition (pop, x, y);
 		e_completion_view_set_width (E_COMPLETION_VIEW (entry->priv->completion_view), dim->width);
 
@@ -507,7 +505,7 @@ e_entry_show_popup (EEntry *entry, gboolean visible)
 		gtk_widget_show (pop);
 
 
-		if (! entry->priv->ptr_grab) {
+		if (getenv ("GAL_E_ENTRY_NO_GRABS_HACK") == NULL && !entry->priv->ptr_grab) {
 			entry->priv->ptr_grab = (0 == gdk_pointer_grab (GTK_WIDGET (entry->priv->completion_view)->window, TRUE,
 									grab_mask, NULL, NULL, GDK_CURRENT_TIME));
 			if (entry->priv->ptr_grab) {
@@ -613,11 +611,11 @@ full_cb (ECompletionView *view, gpointer user_data)
 }
 
 static void
-browse_cb (ECompletionView *view, const gchar *txt, gpointer user_data)
+browse_cb (ECompletionView *view, ECompletionMatch *match, gpointer user_data)
 {
 	EEntry *entry = E_ENTRY (user_data);
-
-	if (txt == NULL) {
+	
+	if (match == NULL) {
 		/* Requesting a completion. */
 		e_entry_start_completion (entry);
 		return;
@@ -629,7 +627,7 @@ browse_cb (ECompletionView *view, const gchar *txt, gpointer user_data)
 	/* If there is no other handler in place, echo the selected completion in
 	   the entry. */
 	if (entry->priv->handler == NULL)
-		e_entry_set_text_quiet (entry, txt);
+		e_entry_set_text_quiet (entry, e_completion_match_get_match_text (match));
 }
 
 static void
@@ -650,7 +648,7 @@ unbrowse_cb (ECompletionView *view, gpointer user_data)
 }
 
 static void
-activate_cb (ECompletionView *view, const gchar *txt, gpointer extra_data, gpointer user_data)
+activate_cb (ECompletionView *view, ECompletionMatch *match, gpointer user_data)
 {
 	EEntry *entry = E_ENTRY (user_data);
 
@@ -661,9 +659,9 @@ activate_cb (ECompletionView *view, const gchar *txt, gpointer extra_data, gpoin
 	e_entry_show_popup (entry, FALSE);
 
 	if (entry->priv->handler)
-		entry->priv->handler (entry, txt, extra_data);
+		entry->priv->handler (entry, match);
 	else
-		e_entry_set_text (entry, txt);
+		e_entry_set_text (entry, match->match_text);
 
 	e_entry_cancel_delayed_completion (entry);
 }
