@@ -354,7 +354,7 @@ e_text_init (EText *text)
 {
 	text->model = e_text_model_new ();
 	text->text = e_text_model_get_text (text->model);
-	
+
 	gtk_object_ref (GTK_OBJECT (text->model));
 	gtk_object_sink (GTK_OBJECT (text->model));
 
@@ -420,6 +420,7 @@ e_text_init (EText *text)
 
 	text->tooltip_timeout = 0;
 	text->tooltip_count = 0;
+	text->tooltip_owner = FALSE;
 
 	text->dbl_timeout = 0;
 	text->tpl_timeout = 0;
@@ -445,6 +446,10 @@ e_text_destroy (GtkObject *object)
 
 	text = E_TEXT (object);
 
+	if (text->tooltip_owner) {
+		e_canvas_hide_tooltip (E_CANVAS(GNOME_CANVAS_ITEM(text)->canvas));
+	}
+
 	if (text->model_changed_signal_id)
 		gtk_signal_disconnect (GTK_OBJECT (text->model), 
 				       text->model_changed_signal_id);
@@ -2304,6 +2309,12 @@ e_text_point (GnomeCanvasItem *item, double x, double y,
 
 	lines = text->lines;
 
+	if ( !lines ) {
+		text->needs_split_into_lines = 1;
+		e_canvas_item_request_reflow (item);
+		return 1;
+	}
+
 	if (text->fill_clip_rectangle) {
 		double clip_width;
 		double clip_height;
@@ -2676,6 +2687,9 @@ static gboolean
 tooltip_event(GtkWidget *tooltip, GdkEvent *event, EText *text)
 {
 	gint ret_val = FALSE;
+	if (GTK_OBJECT_DESTROYED (text)) {
+		return FALSE;
+	}
 	switch (event->type) {
 	case GDK_LEAVE_NOTIFY:
 		e_canvas_hide_tooltip (E_CANVAS(GNOME_CANVAS_ITEM(text)->canvas));
@@ -2697,6 +2711,13 @@ tooltip_event(GtkWidget *tooltip, GdkEvent *event, EText *text)
 	return ret_val;
 }
 
+static void
+tooltip_destroy(GtkWidget *tooltip, EText *text)
+{
+	text->tooltip_owner = FALSE;
+	gtk_object_unref (GTK_OBJECT (text));
+}
+
 static gboolean
 _do_tooltip (gpointer data)
 {
@@ -2877,11 +2898,15 @@ _do_tooltip (gpointer data)
 	gtk_widget_realize (tooltip_window);
 	gtk_signal_connect (GTK_OBJECT(tooltip_window), "event",
 			    GTK_SIGNAL_FUNC(tooltip_event), text);
+	gtk_signal_connect (GTK_OBJECT(tooltip_window), "destroy",
+			    GTK_SIGNAL_FUNC(tooltip_destroy), text);
+	gtk_object_ref (GTK_OBJECT (text));
 
 	e_canvas_popup_tooltip (E_CANVAS(GNOME_CANVAS_ITEM(text)->canvas),
 				tooltip_window,
 				pixel_origin.x - 2 + tooltip_x,
 				pixel_origin.y - 2 + tooltip_y);
+	text->tooltip_owner = TRUE;
 	
 	text->tooltip_timeout = 0;
 	return FALSE;
@@ -2962,7 +2987,7 @@ e_text_event (GnomeCanvasItem *item, GdkEvent *event)
 			e_tep_event.key.time = key.time;
 			e_tep_event.key.state = key.state;
 			e_tep_event.key.keyval = key.keyval;
-
+			
 			// g_print ("etext got keyval \"%s\"\n", gdk_keyval_name (key.keyval));
 
 			/* This is probably ugly hack, but we have to handle UTF-8 input somehow */
@@ -2980,11 +3005,14 @@ e_text_event (GnomeCanvasItem *item, GdkEvent *event)
 			_get_tep(text);
 			ret = e_text_event_processor_handle_event (text->tep, &e_tep_event);
 
-			if (e_tep_event.key.string) g_free (e_tep_event.key.string);
-
 			if (event->type == GDK_KEY_PRESS)
 				gtk_signal_emit (GTK_OBJECT (text), e_text_signals[E_TEXT_KEYPRESS],
 						 e_tep_event.key.keyval, e_tep_event.key.state);
+			
+
+			if (e_tep_event.key.string)
+				g_free (e_tep_event.key.string);
+
 
 			return ret;
 		}
@@ -3175,7 +3203,7 @@ _get_position(EText *text, ETextEventProcessorCommand *command)
 		break;
 
 	case E_TEP_END_OF_BUFFER:
-		new_pos = g_utf8_strlen (text->text, -1);
+		new_pos = strlen (text->text);
 		break;
 
 	case E_TEP_START_OF_LINE:
@@ -3384,6 +3412,7 @@ _get_position(EText *text, ETextEventProcessorCommand *command)
 
 	default:
 		new_pos = text->selection_end;
+		break;
 	}
 
 	new_pos = e_text_model_validate_position (text->model, new_pos);
@@ -3425,6 +3454,7 @@ e_text_command(ETextEventProcessor *tep, ETextEventProcessorCommand *command, gp
 {
 	EText *text = E_TEXT(data);
 	int sel_start, sel_end;
+	gboolean scroll = TRUE;
 
 	switch (command->action) {
 	case E_TEP_MOVE:
@@ -3481,6 +3511,7 @@ e_text_command(ETextEventProcessor *tep, ETextEventProcessorCommand *command, gp
 		if (text->timer) {
 			g_timer_reset(text->timer);
 		}
+		scroll = FALSE;
 		break;
 	case E_TEP_PASTE:
 		e_text_get_selection (text, clipboard_atom, command->time);
@@ -3505,15 +3536,18 @@ e_text_command(ETextEventProcessor *tep, ETextEventProcessorCommand *command, gp
 					GDK_BUTTON_RELEASE_MASK | GDK_POINTER_MOTION_MASK,
 					text->i_cursor,
 					command->time);
+		scroll = FALSE;
 		break;
 	case E_TEP_UNGRAB:
 		gnome_canvas_item_ungrab (GNOME_CANVAS_ITEM(text), command->time);
+		scroll = FALSE;
 		break;
 	case E_TEP_NOP:
+		scroll = FALSE;
 		break;
 	}
 
-	if (!text->button_down) {
+	if (scroll && !text->button_down) {
 		int x;
 		int i;
 		struct line *lines = text->lines;
