@@ -582,6 +582,204 @@ unicode_strrchr (const char *p, gunichar c)
 #endif
 
 
+/* UTF-8 strcasecmp - not optimized */
+
+gint
+g_utf8_strcasecmp (const gchar *s1,
+		   const gchar *s2)
+{
+	gunichar c1, c2;
+
+	g_return_val_if_fail (s1 != NULL && g_utf8_validate (s1, -1, NULL), 0);
+	g_return_val_if_fail (s2 != NULL && g_utf8_validate (s2, -1, NULL), 0);
+
+	while (*s1 && *s2)
+		{
+
+			c1 = g_unichar_tolower (g_utf8_get_char (s1));
+			c2 = g_unichar_tolower (g_utf8_get_char (s2));
+
+			/* Collation is locale-dependent, so this totally fails to do the right thing. */
+			if (c1 != c2)
+				return c1 < c2 ? -1 : 1;
+
+			s1 = g_utf8_next_char (s1);
+			s2 = g_utf8_next_char (s2);
+		}
+
+	if (*s1 == '\0' && *s2 == '\0')
+		return 0;
+
+	return *s1 ? 1 : -1;
+}
+
+
+/* UTF-8 strncasecmp - not optimized */
+
+gint
+g_utf8_strncasecmp (const gchar *s1,
+		    const gchar *s2,
+		    guint n)
+{
+	gunichar c1, c2;
+
+	g_return_val_if_fail (s1 != NULL && g_utf8_validate (s1, -1, NULL), 0);
+	g_return_val_if_fail (s2 != NULL && g_utf8_validate (s2, -1, NULL), 0);
+
+	while (n && *s1 && *s2)
+		{
+
+			n -= 1;
+
+			c1 = g_unichar_tolower (g_utf8_get_char (s1));
+			c2 = g_unichar_tolower (g_utf8_get_char (s2));
+
+			/* Collation is locale-dependent, so this totally fails to do the right thing. */
+			if (c1 != c2)
+				return c1 < c2 ? -1 : 1;
+
+			s1 = g_utf8_next_char (s1);
+			s2 = g_utf8_next_char (s2);
+		}
+
+	if (n == 0 || (*s1 == '\0' && *s2 == '\0'))
+		return 0;
+
+	return *s1 ? 1 : -1;
+}
+
+
+/* UTF-8 strdown */
+
+gchar *
+g_utf8_strdown (gchar *string)
+{
+  gchar *s, *p, *src;
+  gchar buf[6];
+  gunichar c, ctrans;
+  gint sz, sz2;
+  
+  g_return_val_if_fail (string != NULL && g_utf8_validate (string, -1, NULL), NULL);
+  
+  s = string;
+  p = string;
+  while (*s)
+    {
+      c = g_utf8_get_char (s);
+      ctrans = g_unichar_tolower (c);
+      
+      if (! g_unichar_isdefined (ctrans))
+	ctrans = c;
+
+      sz = g_unichar_to_utf8 (ctrans, buf);
+      src = buf;
+
+      /* Failsafe: Use the original character if the transformed
+	 version's UTF-8 representation is larger than the original's.
+	 This should never happen. */
+      if (c != ctrans && (sz2 = g_unichar_to_utf8 (c, NULL)) < sz) {
+	      src = s;
+	      sz = sz2;
+      }
+
+      memcpy (p, src, sz);
+      p += sz;
+
+      s = g_utf8_next_char (s);
+    }
+  
+  *p = '\0';
+  
+  return string;
+}
+
+/* UTF-8 strdup */
+
+gchar *
+g_utf8_strup (gchar *string)
+{
+  gchar *s, *p, *src;
+  gchar buf[6];
+  gunichar c, ctrans;
+  gint sz, sz2;
+  
+  g_return_val_if_fail (string != NULL && g_utf8_validate (string, -1, NULL), NULL);
+  
+  s = string;
+  p = string;
+  while (*s)
+    {
+      c = g_utf8_get_char (s);
+      ctrans = g_unichar_toupper (c);
+      
+      if (! g_unichar_isdefined (ctrans))
+	ctrans = c;
+
+      sz = g_unichar_to_utf8 (ctrans, buf);
+      src = buf;
+
+      /* Failsafe: Use the original character if the transformed
+	 version's UTF-8 representation is larger than the original's.
+	 This should never happen. */
+      if (c != ctrans && (sz2 = g_unichar_to_utf8 (c, NULL)) < sz) {
+	      src = s;
+	      sz = sz2;
+      }
+
+      memcpy (p, src, sz);
+      p += sz;
+
+      s = g_utf8_next_char (s);
+    }
+  
+  *p = '\0';
+  
+  return string;
+}
+
+gchar *
+g_utf8_strtitle (gchar *string)
+{
+  gchar *s, *p, *src;
+  gchar buf[6];
+  gunichar c, ctrans;
+  gint sz, sz2;
+  
+  g_return_val_if_fail (string != NULL && g_utf8_validate (string, -1, NULL), NULL);
+  
+  s = string;
+  p = string;
+  while (*s)
+    {
+      c = g_utf8_get_char (s);
+      ctrans = g_unichar_totitle (c);
+      
+      if (! g_unichar_isdefined (ctrans))
+	ctrans = c;
+
+      sz = g_unichar_to_utf8 (ctrans, buf);
+      src = buf;
+
+      /* Failsafe: Use the original character if the transformed
+	 version's UTF-8 representation is larger than the original's.
+	 This should never happen. */
+      if (c != ctrans && (sz2 = g_unichar_to_utf8 (c, NULL)) < sz) {
+	      src = s;
+	      sz = sz2;
+      }
+
+      memcpy (p, src, sz);
+      p += sz;
+
+      s = g_utf8_next_char (s);
+    }
+  
+  *p = '\0';
+  
+  return string;
+}
+
+
 /* Like g_utf8_get_char, but take a maximum length
  * and return (gunichar)-2 on incomplete trailing character
  */
